import os
import os.path
import json

class Settings:
    Apikey = ""
    Scheme = "http"
    Host = "127.0.0.1"
    Port = 8181
    UseCloudFX = False
    def __init__(self):
        if not os.path.isfile(self.filename()):
            with open(self.filename(), 'w') as outfile:
                json.dump({
                    'Apikey':self.Apikey,
                    'Scheme':self.Scheme,
                    'Host':self.Host,
                    'Port':self.Port,
                    'UseCloudFX':self.UseCloudFX
                    }, outfile, indent=4, sort_keys=True)
        else:
            json_data=open(self.filename()).read()
            data = json.loads(json_data)
            self.__dict__.update(data)

    def filename(self):
        fn = os.getcwd()
        fn = os.path.join(fn, 'settings.json')
        return fn

    def save(self):
        with open(self.filename(), 'w') as outfile:
            json.dump({
                'Apikey':self.Apikey,
                'Scheme':self.Scheme,
                'Host':self.Host,
                'Port':self.Port,
                'UseCloudFX':self.UseCloudFX
                }, outfile, indent=4, sort_keys=True)

    def getUrl(self, path):
        if self.UseCloudFX:
            return "https://api.unifiedfx.com/" + path
        else:
            url = "{0}://{1}:{2}/AutomationFX/api/{3}"
            url = url.format(self.Scheme,self.Host,self.Port, path)
            return url




from requests.auth import AuthBase

class APIKeyAuth(AuthBase):
    """Attaches APIKey Authentication to the given Request object."""
    apikey = ""
    def __init__(self, apikey):
        # setup any auth-related data here
        self.apikey = apikey

    def __call__(self, r):
        # modify and return the request
        r.headers['Apikey'] = self.apikey
        return r



import time
from ..api.client import Client
from odata import ODataService
from ..models.phones import Phone
from ..auth import APIKeyAuth
from ..settings import Settings

api = None

def getClient():
    global api
    if api is None:
        api = Client()
    return api

def call(source, number,line='1'):
    """Call number form source"""
    name = source.Name
    print(("Calling {0} from {1}".format(number, name)))
    return getClient().post("cti/calls/{0}/{1}".format(name, number), None, {'address': line})

def answer(source):
    print(("Answering call on {0}".format(source.Name)))
    return operation(source,'Answer')

def drop(source):
    print(("Dropping call on {0}".format(source.Name)))
    return operation(source,'Drop')

def callInfo(source):
    print(("Getting call info on {0}".format(source.Name)))
    return getClient().get("cti/calls/legacy/{0}".format(source.Name))

def startMonitor(source, supervisor):
    """Start a Call Monitoring session on source phone and send media to supervisor phone"""
    name = source.Name
    supervisorName = supervisor.Name
    print(("Monitoring call on {0} from {1}".format(name, supervisorName)))
    return getClient().post("cti/calls/{0}/monitor/{1}".format(name, supervisorName), None)

def stopMonitor(source):
    """Stop the Call Monitoring Session on source phone"""
    name = source.Name
    print(("Stopping call monitoring session on {0}".format(name)))
    return getClient().delete("cti/calls/{0}/monitor".format(name), None)

def hold(source):
    """Place the active call on the source phone on hold"""
    print(("Holding call on {0}".format(source.Name)))
    return operation(source,'Hold')

def unhold(source):
    print(("Unholding call on {0}".format(source.Name)))
    return operation(source,'UnHold')

def transfer(source, number = None):
    if number is not None:
        print(("Direct Transfer from {0} to {1}".format(source.Name, number)))
        return getClient().post("cti/calls/{0}/transfer/{1}".format(source.Name,number), None)
    else:
        print(("Consult Transfer on {0}".format(source.Name)))
        return operation(source,'Transfer')

def conference(source, number= None):
    if number is not None:
        print(("Direct Conference from {0} to {1} ".format(source.Name, number)))
        return getClient().post("cti/calls/{0}/conference/{1}".format(source.Name,number), None)
    else:
        print(("Consult Conference on {0}".format(source.Name)))
        return operation(source,'Conference')
        

def offhook(source):
    print(("Offhook on {0}".format(source.Name)))
    return operation(source,'OffHook')

def operation(source, op):
    name = source.Name
    return getClient().post("cti/calls/{0}/{1}".format(name,op), None)


def sendDigits(source, digits):
    name = source.Name
    print(("Send Digits '{0}' on {1}".format(digits, name)))
    return getClient().post("cti/calls/{0}/digits/{1}".format(name,digits), None)

def sendData(source, data):
    name = source.Name
    return getClient().post("cti/data/{0}".format(name), data=data)

def sendUri(source, uri):
    name = source.Name
    return getClient().postString("cti/execute/{0}".format(name), uri)

def pause(delay):
    print(("Pausing for {0} seconds".format(delay)))
    time.sleep(delay)

def macro(source, macro):
    name = source.Name
    print(("Sending macro '{0}' to {1}".format(macro, name)))
    return getClient().postString("phones/macro/{0}".format(name),macro)

def sendMessage(message, destination):
    """
    Send Message
    ============
    Send a text/audio message to a group of phone using an existing message and filter defined in the AutomationFX admin interface
    - message (the name of the message to send)
    - destination (the name of the filter to send the message too)
    """
    return getClient().post("messages/{0}/send".format(message), None, params={'filter':destination})

def sendRawMessage(message, source):
    return getClient().post("messages/send", data=message, params={'filter':source})

def migrate(oldphone, newphone, newmodel, newprotolcol = "SIP", enableactivationcode = False):
    """
    Initiate a Phone Migration
    ============
    Intiates a phone migration in CUCM 
    - oldphone(string) : Name of the old phone to migrate from (required).
    - newphone(string) : Name of the new phone, if not provided a random name will be generated.
    - newmodel(string) : New phone Model, This is not required if new name (newphone) is provided and the new phone exist in CUCM with the correct model. 
    - newprotocol(string) - New phones protocol (if not provided defaults to SIP)
    - enableactivationcode(boolean) - Generates an Activation Code. Requires UCM 12.5+ and Device Default 'Onboarding Method' for model should be configured to 'Activation Code'     
    """
    model = {'oldName': oldphone, 'newName': newphone, 'newModel': newmodel, 'newProtocol': newprotolcol, 'createActivationCode' :enableactivationcode}
    return getClient().post("migrate", data=model)

def listResource(resource, search=None, fields=None):
    params = {'detailed':'true'}
    if search is not None:
        params.update({'search':search})
    if fields is not None:
        params.update({'fields':fields})
    return getClient().get("axl/{0}".format(resource), params=params)

def getResource(resource, id):
    return getClient().get("axl/{0}/{1}".format(resource,id))

def addResource(resource, data):
    return getClient().post("axl/{0}".format(resource), data=data)

def updateResource(resource, id, data):
    return getClient().put("axl/{0}/{1}".format(resource,id),data)

def deleteResource(resource, id):
    return getClient().delete("axl/{0}/{1}".format(resource,id))

def sqlQuery(query):
    return getClient().postString("axl/sql/query", query)

def sqlUpdate(update):
    return getClient().postString("axl/sql/update", update)

def queryPhone():
    settings = Settings()
    Service = ODataService(settings.getUrl('oData'), reflect_entities=False, base=Phone, auth=APIKeyAuth(settings.Apikey))
    return Service.query(Phone)

def findPhone(param):
    settings = Settings()
    Service = ODataService(settings.getUrl('oData'), reflect_entities=False, base=Phone, auth=APIKeyAuth(settings.Apikey))
    return queryPhone().filter(param).first()



from odata import ODataService
from .models.phones import Phone
from .auth import APIKeyAuth
from .settings import Settings
from .actions import *

__all__ =[
    'queryPhone',
    'findPhone',
    'Phone',
    'call',
    'callInfo',
    'answer',
    'drop',
    'hold',
    'unhold',
    'transfer',
    'conference',
    'sendDigits',
    'sendData',
    'sendUri',
    'pause',
    'macro',
    'sendMessage',
    'sendRawMessage',
    'startMonitor',
    'stopMonitor',
    'migrate',
    'listResource',
    'getResource',
    'addResource',
    'updateResource',
    'deleteResource',
    'sqlQuery',
    'sqlUpdate']



import json
import requests
from ..settings import Settings

class Client:
    settings = Settings()
    def __init__(self,session=None):
        if session is None:
            self.session = requests.Session()
        else:
            self.session = session
        self.session.headers.update({'apikey':self.settings.Apikey})

    def get(self, path, params=None):
        url = self.settings.getUrl(path)
        response = self.session.get(url, params=params)
        return response.json()
    
    def post(self, path, data, params=None):
        url = self.settings.getUrl(path)
        headers = {
            'Content-Type': 'application/json',
        }
        data = json.dumps(data)
        response = self.session.post(url, data=data, headers=headers, params=params)
        return response.json()

    def postString(self, path, data, params=None):
        url = self.settings.getUrl(path)
        headers = {
            'Content-Type': 'text/plain',
        }
        response = self.session.post(url, data=data, headers=headers, params=params)
        try:
            jdata = response.json() 
        except (ValueError, e):
            return
        return jdata
    
    def put(self, path, data, params=None):
        url = self.settings.getUrl(path)
        headers = {
            'Content-Type': 'application/json',
        }
        data = json.dumps(data)
        response = self.session.put(url, data=data, headers=headers, params=params)
        return response.json()
    
    def delete(self, path, params=None):
        url = self.settings.getUrl(path)
        response = self.session.delete(url, params=params)
        return response.json()


import cmd, sys, argparse
import six
import os.path
from .settings import Settings
from .actions import *

def main():
    AutomationFXShell().cmdloop()

def input_polyphil(value):
    if six.PY3:
        return input(value)
    else:
        return raw_input(value)	

def query_yes_no(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".
    """	
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        if six.PY3:
            choice=input().lower()
        else:
            choice=raw_input().lower()
		
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")

class AutomationFXShell(cmd.Cmd):
    intro = 'Welcome to the AutomationFX cli. Type help or ? to list commands.\n'
    prompt = 'AFX> '
    pwd = None
    settings = Settings()

    def __init__(self):
        cmd.Cmd.__init__(self)
        if not self.settings.Apikey:
            print("No Apikey, running setup")
            self.do_setup(None)

    def do_setup(self,arg):
        'Setup the connection to AutomationFX'
        if query_yes_no("Set Apikey?"):
            self.settings.Apikey = input_polyphil("Apikey: ") 
        self.settings.UseCloudFX = query_yes_no("Use CloudFX?")
        if not self.settings.UseCloudFX and query_yes_no("Change AutomationFX Host ({0})?".format(self.settings.Host)):
            self.settings.Host = input_polyphil("AutomationFX Host: ")
        if not self.settings.UseCloudFX and query_yes_no("Change AutomationFX Port ({0})?".format(self.settings.Port)):
            self.settings.Port = int(input_polyphil("AutomationFX Port: "))
        if not self.settings.UseCloudFX and query_yes_no("Change AutomationFX Scheme ({0})?".format(self.settings.Scheme)):
            scheme = input_polyphil("AutomationFX Scheme: ")
            scheme = scheme.lower()
            if scheme != "http" and scheme != "https":
                print("Invalid scheme (only 'http' and 'https' allowed)")
            else:
                self.settings.Scheme = scheme
        print("AutomationFX Setup:")
        print(("\tApikey: {0}".format(self.settings.Apikey)))
        print(("\tUse CloudFX: {0}".format(self.settings.UseCloudFX)))
        if not self.settings.UseCloudFX:
            print(("\tHost: {0}".format(self.settings.Host)))
            print(("\tPort: {0}".format(self.settings.Port)))
            print(("\tScheme: {0}".format(self.settings.Scheme)))
        if query_yes_no("Save Settings?"):
            self.settings.save()

    def intTryParse(self, value):
        try:
            return int(value), True
        except ValueError:
            return value, False

    def getArgs(self, arg, number=1):
        parts = arg.split()
        if len(parts) < number:
            print("Invalid number of arguments")
            return
        return parts

    def getPhone(self, arg, number=1):
        if self.pwd:
            parts = arg.split()
            # result = parts.insert(0, self.pwd.DN)
            result = ([self.pwd.DN] + parts)
            return self.pwd, result
        parts = self.getArgs(arg, number=1)
        if not parts:
            return
        sourceNum = self.intTryParse(parts[0])
        if not sourceNum[1]:
            print(("Argument ({0}) is not a number".format(parts[0])))
            return
        phone = findPhone(Phone.DN == parts[0])
        if not phone:
            print(("Extension/DN '{0}' not found".format(parts[0])))
            return
        return phone, parts

    def do_call(self, arg):
        'Place a call from a source phone extension to any number "call 50005 10134"'
        data = self.getPhone(arg, 2)
        if not data[0]:
            return
        call(data[0], data[1][1])

    def do_connect(self, arg):
        'Connect (Call and Answer) a call between the source and destiantion phones "connect 50005 10134"'
        data = self.getPhone(arg, 2)
        if not data[0]:
            return
        data2 = self.getPhone(data[1][1], 1)
        if not data2[0]:
            return
        call(data[0], data[1][1])
        answer(data2[0])

    def do_answer(self, arg):
        'Answer any call on the phone "answer 50005"'
        data = self.getPhone(arg, 1)
        if not data[0]:
            return
        answer(data[0])

    def do_drop(self, arg):
        'Drop all calls on the phone "drop 50005"'
        data = self.getPhone(arg, 1)
        if not data[0]:
            return
        drop(data[0])

    def do_hold(self, arg):
        'Hold the current call on the phone "hold 50005"'
        data = self.getPhone(arg, 1)
        if not data[0]:
            return
        hold(data[0])

    def do_unhold(self, arg):
        'Unhold the current call on the phone "hold 50005"'
        data = self.getPhone(arg, 1)
        if not data[0]:
            return
        unhold(data[0])

    def do_transfer(self, arg):
        'Initiate/Complete a consult transfer on the current call on the phone "transfer 50005"'
        data = self.getPhone(arg, 1)
        if not data[0]:
            return
        transfer(data[0])

    def do_conference(self, arg):
        'Initiate/Complete a consult conference on the current call on the phone "conference 50005"'
        data = self.getPhone(arg, 1)
        if not data[0]:
            return
        conference(data[0])

    def do_offhook(self, arg):
        'Go off-hook on the primary line on the phone "offhook 50005"'
        data = self.getPhone(arg, 1)
        if not data[0]:
            return
        offhook(data[0])

    def do_digits(self, arg):
        'Send digits to the phone "digits 50005 123"'
        data = self.getPhone(arg, 2)
        if not data[0]:
            return
        digits = self.intTryParse(data[1][1])
        if not digits[1]:
            print(("Argument ({0}) is not a number".format(data[1][1])))
            return
        sendDigits(data[0], data[1][1])

    def do_uri(self, arg):
        'Send a uri/url to the phone "uri 50005 Key:Soft1"'
        data = self.getPhone(arg, 2)
        if not data[0]:
            return
        sendUri(data[0], data[1][1])

    def do_macro(self, arg):
        'Send a macro to the phone "macro 50005 Key:Line1|Key:KeyPad1"'
        data = self.getPhone(arg, 2)
        if not data[0]:
            return
        macro(data[0], data[1][1])

    def do_cd(self,arg):
        'Change Device, set the current scope to the specified device extension "cd 50005"'
        if arg == '..' or arg == '/':
            self.prompt = 'AFX> '
            self.pwd = None
            return
        phone = findPhone(Phone.DN == arg)
        if not phone:
            print(("Extension/DN '{0}' not found".format(arg)))
            return
        self.pwd = phone
        self.prompt = 'AFX/{0}>'.format(phone.DN)
    
    def do_run(self,arg):
        'Run a python script "run test.py"'
        filename = arg or "test"
        if os.path.isfile(filename):
            print(("Running: {0}", format(filename)))
            exec(compile(open(filename).read(), filename, 'exec'))
            return 
        filename = filename+'.py'
        if os.path.isfile(filename):
            print(("Running: {0}".format(filename)))
            exec(compile(open(filename).read(), filename, 'exec'))
            return
        print(("could not find file to run: {0}".format(filename)))            

if __name__ == "__main__":
    main()